// ****************
// Datafile.h file
// *****************
/*

DataFile 

License: GNU General Public License
firs version :
Cordier Yves 24/04/2016

V1.00 : 24/04/2016

	This unit is use to get easily numeric values
	from a text file. It could read
	positive integer
	relative integer
	reals either notation xx.yy and xx.yyEzz
	where xx and zz are a relative (positive or negative with a -)
	in an otherr hand, it permises to include
	end of line comments (preceded a char selected by setComment(int) fonction)
	by default ther is no comment char (setComment(-1))
	
	see the example program
	
V2.00 : 20/03/2020
	extensions of possibilities of this librarie
	more easier code to open close ... files
	ability to write floats double with format
	in one word a pascal style file function
	
V2.01 : 23/03/2020
	correction of a minor bug in printing scientific mode 
	(add rounding of decimal part)
	
V2.10 : 25/03/2020
	normalizing name of function according camelBack spelling
	accept the function readln() with no argument ie readln(NULL)
	accept readln with a char[] in argument (as well as String)

V2.11 : 02/04/2020
	correction of a pernicious bug when using 0x00 as comment character
		
*/

#ifndef __DATAFILE_H__
#define __DATAFILE_H__
#ifndef __SD_H__  // automatic include of SD librairie if non included before
#include <SD.h>
#endif

#define _version "v2.11"

class DataFile : public SDFile
{
protected:
	int carComment = 0 ; // The comment begining charact. 0 if none
public:
	// Constructors
	DataFile(SdFile f, const char *name);     // wraps an underlying SdFile
	DataFile(void);  // 'empty' constructor
	//
	// in order to open a file more simply
	boolean open(const char *filename, uint8_t mode = FILE_READ); // open specific file assume that SD corectely linked with
	// SD.begin(pinNumber) before use
	boolean open(const String &filename, uint8_t mode = FILE_READ) // to be able to work with String
	{
		return open( filename.c_str(), mode );
	};

	// new methodes to be able to act with numeric data files with comments
	//
	int peekComment(); // peek with taking care of comments
	void setComment(int c); // sets the comment character -1 if none
	int readComment();// read but skipping the comments
	void skipEoln(); // skip until end of line
	boolean eof(); // end of file
	void  skipNonNumeric(boolean minus); // skip non numeric parts of file including comments accepting minus or not
	unsigned long readPos(); // read next positive value 0 if none
	long  readLong(); // read next positive or negative value 0 if none
	float  readFloat(); // read next float value
	float readFloatln(); // read nest float value and go to next line (0 if error)
	long readLongln();// read nest positive or negative integer and go to next line (0 if error)
	unsigned long readPosln(); // read nest positive integer and go to next line (0 if error)
	
	void printFloat(double d, int n=8, int dec=2); // print a double with ndigits and d digits after point default 8,2
	void printFloatln(double d, int n=8, int dec=2); // print a double with ndigits and d digits after point default 8,2
	void printScienti(double d, int dec ); // print a double scientific methode with n digits decimal 

	void printScientiln(double d, int dec ); // print a double scientific methode with n digits decimal 

	boolean readln(String &S); // reading a line or nothing until end of line result a ability to read
	boolean readln(char S[]); // idem result a char[]
	boolean readln(); // skip until end of line
	
private:
	unsigned long update(); // read positive number from position of file (0 if error)
};
#endif

